{
*****************************************************************************
*  Name    : USB StopWatch                                                  *
*  Author  : Hiroyuki Iizuka                                                *
*  Date    : 2010/03/20                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Program USBStopWatch

'---- configuration ---------------------------------------------------------
Device = 18F4550
Clock  = 48

Config
   USBDIV  = 1, 
   PLLDIV  = 5,
   CPUDIV  = OSC1_PLL2,
   FOSC    = HSPLL_HS,
   MCLRE   = OFF

'---- include file ----------------------------------------------------------
Include "Convert.bas"
Include "EEPROM.bas"

'---- interrupt -------------------------------------------------------------
#option org_reset     = $2000
#option vector_isr_hi = $2008
#option vector_isr_lo = $2018

Const ipLow  = 1
Const ipHigh = 2

Const High_priority = 1
Const Low_priority  = 0

Dim Interrupt_HIGH_enable As INTCON.booleans(7)
Dim Interrupt_LOW_enable  As INTCON.booleans(6)
Dim Interrupt_priority    As RCON.booleans(7)

'------------ Push Switch ---------------------------------------------------
Dim SW_START As PORTB.4  ' SW1
Dim SW_SPLIT As PORTA.3  ' SW2 

Const SW_ON      = 0
Const SW_OFF     = 1

Const SW_ENABLE  = 0
Const SW_DISABLE = 1

Dim SW_START_status As Byte
Dim SW_SPLIT_status As Byte

Dim time_status    As Byte
Dim display_status As Byte

Const RUN   = 0
Const STOP  = 1
Const HOLD  = 1

'------------ Mechanical RELAY ---------------------------------------------
Dim RELAY As PORTA.5

Const RELAY_ON  = 0
Const RELAY_OFF = 1

'------------ 3mm LED ------------------------------------------------------
Dim LED1 As PORTB.5
Dim LED2 As PORTB.3
Dim LED3 As PORTA.2


'---- 7segment LED ---------------------------------------------------------
Dim LED_DEV6     As PORTC.0
Dim LED_DEV5     As PORTC.1
Dim LED_DEV4     As PORTC.2
Dim LED_DEV3     As PORTE.0
Dim LED_DEV2     As PORTE.1
Dim LED_DEV1     As PORTE.2

Dim LED_SEG_PORT As PORTD
Dim LED_SEG_A    As PORTD.0
Dim LED_SEG_B    As PORTD.1
Dim LED_SEG_C    As PORTD.2
Dim LED_SEG_D    As PORTD.3
Dim LED_SEG_E    As PORTD.4
Dim LED_SEG_F    As PORTD.5
Dim LED_SEG_G    As PORTD.6
Dim LED_SEG_DP   As PORTD.7

Const Digit_N  = $00   ' Blank
Const Digit_0  = $3F
Const Digit_1  = $06
Const Digit_2  = $5B
Const Digit_3  = $4F
Const Digit_4  = $66
Const Digit_5  = $6D
Const Digit_6  = $7D
Const Digit_7  = $07
Const Digit_8  = $7F
Const Digit_9  = $6F
Const Digit_DP = $80

Const LED_ON        = 1
Const LED_OFF       = 0

Dim led_val1 As Byte
Dim led_val2 As Byte
Dim led_val3 As Byte
Dim led_val4 As Byte
Dim led_val5 As Byte
Dim led_val6 As Byte
Dim led_pos  As Byte

'-------- TMR0 ------------------------------------------
Dim Timer0_overflowd             As INTCON.booleans(2)
Dim Timer0_interrupt_enable      As INTCON.booleans(5)
Dim Timer0_priority              As INTCON2.bits(2)
Dim Timer0_enable                As T0CON.booleans(7)
Dim Timer0_mode_8bit             As T0CON.booleans(6)
Dim Timer0_use_external_clock    As T0CON.booleans(5)
Dim Timer0_prescale_NOT_assign   As T0CON.booleans(3)
Dim Timer0_prescale_bit2         As T0CON.bits(2)
Dim Timer0_prescale_bit1         As T0CON.bits(1)
Dim Timer0_prescale_bit0         As T0CON.bits(0)

{
'-------- TMR1 ------------------------------------------
Dim Timer1_overflowd             As PIR1.booleans(0) 
Dim Timer1_interrupt_enable      As PIE1.booleans(0) 
Dim Timer1_prescale_bit0         As T1CON.bits(4)
Dim Timer1_prescale_bit1         As T1CON.bits(5)
Dim Timer1_use_external_clock    As T1CON.booleans(1)
Dim Timer1_enable                As T1CON.booleans(0)
Dim Timer1_priority              As IPR1.bits(0)
}

'-------- TMR2 ------------------------------------------
Dim Timer2_overflowd             As PIR1.booleans(1) 
Dim Timer2_interrupt_enable      As PIE1.booleans(1) 
Dim Timer2_prescale_bit0         As T2CON.bits(0)
Dim Timer2_prescale_bit1         As T2CON.bits(1)
Dim Timer2_enable                As T2CON.booleans(2)
Dim Timer2_postscale_bit0        As T2CON.bits(3)
Dim Timer2_postscale_bit1        As T2CON.bits(4)
Dim Timer2_postscale_bit2        As T2CON.bits(5)
Dim Timer2_postscale_bit3        As T2CON.bits(6)
Dim Timer2_priority              As IPR1.bits(1)

'-------- A/D converter ---------------------------------
Dim ADconfig3 As ADCON1.3
Dim ADconfig2 As ADCON1.2
Dim ADconfig1 As ADCON1.1
Dim ADconfig0 As ADCON1.0

'-------- comparator ------------------------------------
Dim Comparator_mode2 As CMCON.bits(2)
Dim Comparator_mode1 As CMCON.bits(1)
Dim Comparator_mode0 As CMCON.bits(0)

'-------- PORTB -----------------------------------------
Dim PortB_pullups_disable As INTCON2.booleans(7)

'---- stop watch ----------------------------------------
Dim time_count     As LongWord


'===============================================================================
' interrupt priority HIGH
'===============================================================================
Interrupt OnTimerHigh(ipHigh)
  '---- TMR2 overflow ----------
  If Timer2_overflowd Then
    If time_status = RUN Then
      time_count = time_count + 1
    End If
    Timer2_overflowd = false
  End If
End Interrupt


'===============================================================================
' interrupt priority LOW
'===============================================================================
Interrupt OnTimerLow(ipLow)
  '--- TMR0 overflow (7SEG_LED storobe)   2 msec
  If Timer0_overflowd Then
    TMR0L = 69
    Select led_pos
      Case 1
        LED_DEV6 = LED_OFF
        LED_SEG_PORT = led_val1
        LED_DEV1 = LED_ON
        led_pos = led_pos + 1
      Case 2
        LED_DEV1 = LED_OFF
        LED_SEG_PORT = led_val2
        LED_DEV2 = LED_ON
        led_pos = led_pos + 1
      Case 3
        LED_DEV2 = LED_OFF
        LED_SEG_PORT = led_val3
        LED_DEV3 = LED_ON
        led_pos = led_pos + 1
      Case 4
        LED_DEV3 = LED_OFF
        LED_SEG_PORT = led_val4
        LED_DEV4 = LED_ON
        led_pos = led_pos + 1
      Case 5
        LED_DEV4 = LED_OFF
        LED_SEG_PORT = led_val5
        LED_DEV5 = LED_ON
        led_pos = led_pos + 1
      Case 6
        LED_DEV5 = LED_OFF
        LED_SEG_PORT = led_val6
        LED_DEV6 = LED_ON
        led_pos = 1
    EndSelect

    Timer0_overflowd = false   ' Clear interrupt flag
  End If
End Interrupt


'===============================================================================
' decode number -> 7segment-LED
'===============================================================================
Function led7seg_decode_number( num As Byte ) As Byte
  Select num
    Case 0  result = Digit_0
    Case 1  result = Digit_1
    Case 2  result = Digit_2
    Case 3  result = Digit_3
    Case 4  result = Digit_4
    Case 5  result = Digit_5
    Case 6  result = Digit_6
    Case 7  result = Digit_7
    Case 8  result = Digit_8
    Case 9  result = Digit_9
  Else
    result = Digit_N  
  EndSelect
End Function


'===============================================================================
'  number -> led_val1 to led_val6
'===============================================================================
Sub decode_number( num As LongWord )
  Dim tmp_val(6) As Byte
  Dim tmp_count As LongWord
  Dim i As Byte

  tmp_count = num / 5 
  tmp_val(0) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(1) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(2) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(3) = Byte(tmp_count Mod 6  ) 
  tmp_count  = tmp_count / 6
  tmp_val(4) = Byte(tmp_count Mod 10 ) 
  tmp_count  = tmp_count / 10
  tmp_val(5) = Byte(tmp_count Mod 6  )
  
  For i = 5 To 1 Step -1
    If tmp_val(i) = 0 Then
      tmp_val(i) = 10
    Else
      Break
    End If
  Next

  led_val1 = led7seg_decode_number( tmp_val(0) )
  led_val2 = led7seg_decode_number( tmp_val(1) )
  led_val3 = led7seg_decode_number( tmp_val(2) )
  led_val4 = led7seg_decode_number( tmp_val(3) )
  led_val5 = led7seg_decode_number( tmp_val(4) )
  led_val6 = led7seg_decode_number( tmp_val(5) )
End Sub


'*******************************************************************************
' Program main
'*******************************************************************************
PortB_pullups_disable  = true

ADconfig0 = 1  ' PortA Digital I/O
ADconfig1 = 1
ADconfig2 = 1 
ADconfig3 = 1

Comparator_mode2 = 1 ' compatator off
Comparator_mode1 = 1 ' compatator off
Comparator_mode0 = 1 ' compatator off
 

Input(SW_START)
Input(SW_SPLIT)

Output(RELAY)

Output(LED1)
Output(LED2)
Output(LED3)

Output(LED_DEV6)
Output(LED_DEV5)
Output(LED_DEV4)
Output(LED_DEV3)
Output(LED_DEV2)
Output(LED_DEV1)
   
Output(LED_SEG_A)
Output(LED_SEG_B)
Output(LED_SEG_C)
Output(LED_SEG_D)
Output(LED_SEG_E)
Output(LED_SEG_F)
Output(LED_SEG_G)
Output(LED_SEG_DP)

LED_DEV6 = LED_OFF
LED_DEV5 = LED_OFF
LED_DEV4 = LED_OFF
LED_DEV3 = LED_OFF
LED_DEV2 = LED_OFF
LED_DEV1 = LED_OFF

led_val1 = Digit_8
led_val2 = Digit_8
led_val3 = Digit_8
led_val4 = Digit_8
led_val5 = Digit_8
led_val6 = Digit_8
led_pos  = 1  


'---- TMR0(8bitMODE) initialize --------------
Timer0_prescale_bit2 = 1   ' PreScaler 1:128
Timer0_prescale_bit1 = 1   ' PreScaler 1:128
Timer0_prescale_bit0 = 0   ' PreScaler 1:128
Timer0_prescale_NOT_assign = false
Timer0_mode_8bit           = true
Timer0_interrupt_enable    = true
Timer0_use_external_clock  = false
Timer0_enable              = true
Timer0_priority            = Low_priority


'---- TMR2(8bit) initialize ------------------
Timer2_interrupt_enable    = true 
Timer2_prescale_bit0       = 1 ' 1:16
Timer2_prescale_bit1       = 1
Timer2_postscale_bit0      = 1 ' 1:10
Timer2_postscale_bit1      = 0
Timer2_postscale_bit2      = 0
Timer2_postscale_bit3      = 1
Timer2_enable              = true
PR2 = 149
Timer2_priority            = High_priority


'---- interrpt -------------------------------
Enable(OnTimerHigh)
Enable(OnTimerLow)
Interrupt_priority    = true
Interrupt_HIGH_enable = true
Interrupt_LOW_enable  = true
 
 
LED1 = LED_OFF
LED2 = LED_OFF

 
RELAY = RELAY_OFF 
LED3 = LED_OFF 

DelayMS(1000)


time_count = 0
  
time_status    = STOP
display_status = RUN

SW_START_status = SW_ENABLE 
SW_SPLIT_status = SW_ENABLE 

While( true )
  '---- START / STOP switch ---------------
  If SW_START = SW_ON Then
    If SW_START_status = SW_ENABLE Then
      If time_status = STOP Then
        time_status = RUN
        RELAY = RELAY_ON
        LED3 = LED_ON
      Else
        time_status = STOP
        RELAY = RELAY_OFF
        LED3 = LED_OFF
      End If
      SW_START_status = SW_DISABLE
    End If
  End If  
    
  If SW_START_status = SW_DISABLE Then  
    If SW_START = SW_OFF Then
      SW_START_status = SW_ENABLE
    End If
  End If    
    
  '---- SPLIT / RESET switch ---------------
  If SW_SPLIT = SW_ON Then
    If SW_SPLIT_status = SW_ENABLE Then
      If display_status = RUN Then
        If time_status = STOP Then
          time_count = 0
          LED2 = LED_OFF
        Else
          display_status = HOLD
          LED2 = LED_ON
        End If
      Else
        display_status = RUN
        LED2 = LED_OFF
      End If
      SW_SPLIT_status = SW_DISABLE
    End If
  End If  
    
  If SW_SPLIT_status = SW_DISABLE Then  
    If SW_SPLIT = SW_OFF Then
      SW_SPLIT_status = SW_ENABLE
    End If
  End If
    
    
  '---- display time ----------------------  
  If display_status = RUN Then
    decode_number( time_count )
  End If
Wend


'--------- E N D  O F  P R O G R A M ------------------------------------------- 
 
